/***************************************************************************
                  engine.h  -  header file for engine.cpp
                             -------------------
    begin                : Sat Oct 19 2002
    copyright            : (C) 2002 by Andrew D. Balsa
    email                : andrebalsa@mailingaddress.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* Note: Globals are interspersed with defines, I have to clean this up!
 * 	 ADB, August 2002
 */

#ifndef ENGINE_H
#define ENGINE_H

void panic(char *);
void fatal(char *);

int enemymove();
int mymove();

void initgame();
void score();

/*--------------  Defines  ------------------*/

#define ASSERT 0        /* flag for whether bnassert() macros should be */
                        /* expanded */

#define BIGU 10000      /* a number higher than any meaningful "urgency" */
                        /* or move importance */

#define PLAY_MOVE (0)   /* code for normal play */
#define RESIGN (-2)     /* codes for resigning */
#define PASS (-1)       /* passing moves */
#define BOTHPASS (-3)   /* code for both sides passed, game is over */

#define EDGE 25         /* max # intersections on the edge of a go board*/

#define NPLAYERS 2      /*# players (hence # copies to keep of score &c.)*/

/*Return the absolute value of i.*/
#define abs(i) ((i)>=0?(i):-(i))

/*Return TRUE if (x,y) corresponds to a point on the board.*/
#define on1board(x) (0<=(x)&&(x)<thegame.boardSize)
#define onboard(x,y) (on1board(x)&&on1board(y))

/*Return TRUE if (x,y) is on the edge of the board.*/
#define onedge1(x) (0==(x)||thegame.boardSize-1==(x))
#define onedge(x,y) (onedge1(x)||onedge1(y))

/*codes for players; and a code for something (e.g. a point on the board) */
/* which belongs to neither*/
#define BLACK 0
#define WHITE 1
#define EMPTY 2
/*flags in shape table entries for types of points*/
#define F_BLACK 1       /*the point has a black stone*/
#define F_WHITE 2       /*the point has a white stone*/
#define F_EMPTY 4       /*the point is empty*/
#define F_OFF   8       /*the point is off the board*/
int flookup[]=  /*sets of flags corresponding to codes*/
     {
	 F_BLACK, F_WHITE, F_EMPTY
     };

/*Return TRUE if a move violates ko.*/
#define ko(x,y) (thegame.kox==(x)&&thegame.koy==(y))

/*Return a letter corresponding to column x; 'i' is omitted from letter */
/* sequence by tradition.*/
#define lettercol(x) (                                                  \
		      (0<=(x)&&(x)<=7) ? 'a'+(x) : (7<(x)&&(x)< thegame.boardSize) ? 'a'+(x)+1 :  \
		    		'@')
//				This character replaces the call
//		                panic("illegal lettercol()") )
//				to avoid compilation errors with g++

/*Convert any uppercase letter to lowercase.*/
#define lowercase(c) ('A'<=(c)&&(c)<='Z'?(c)-'A'+'a':(c))

/*Return a string naming BLACK or WHITE.*/
char whitename[]="White", blackname[]="Black";
#define pname(c) (BLACK==(c)?blackname:WHITE==(c)?whitename:\
		          (panic("illegal input to pname()"),""))

/*Return TRUE if a move would leave the group it creates or attaches to */
/* in atari or dead.*/
#define intoatari(x,y) (subj_lib(x,y)<=1)

/*a macro to abort the program if a test is not successful*/
#if ASSERT
#define bnassert(q) ((q)?1:(fprintf(stderr, \
				  "\n?!panic--failed bnassert() @ %s %d.\n", \
				   __FILE__, __LINE__), fflush(stderr),exit(1)))
#else
#define bnassert(q)
#endif /* ASSERT */

/*ASCII character codes*/
#define TAB 8

/*--------------  Globals  ------------------*/

bool debugattack = false;   /* flag for printing debugging messages during */
                            /* move generation */

/* a struct for holding information on every square of the board */
typedef int board[EDGE][EDGE];

/*current position of stones on the board*/
board theboard;

/* List of vertices where handicap stones have been placed */
string handicapList;

/*other information about the game*/

typedef struct group
     {
	 int	color,          /*the color of stones in this group*/
	        nliberties,     /*the number of liberties that this group has*/
	        nstones,        /*the number of stones in this group*/
	        x, y;           /*one point in this group*/
     } group;

#define MAXGROUPS 125     /* On a 19 x 19 board, the maximum number of groups */
group groups[MAXGROUPS]; /* (EDGE*EDGE was a lazy upper bound; the number */
                         /* of groups must be less than number of points). */
int ngroups;

/*a variable for debugging -- records the number of the pattern the */
/* program thinks it has matched, so when the program makes a bizarre */
/* move and claims that this helps it make shape, I can see what pattern */
/* has gone wrong*/
int patnum;

/*Where did the enemy last move?  This is used to help decide which patterns */
/* are most important (try to leech off enemy's greater expertise).*/
int lex, ley;

/*
a table of moves the heuristics are ambivalent about, for
random selection
*/
int goodmoves[2*EDGE*EDGE];
int *pgoodmoves;        /* &next free space in goodmoves[]*/


/* -------- Functions & Variables --------- */

int subj_lib(int x, int y, int color = BLACK);

void place_fixed_handicap();
void place_handicap_stone (int x, int y);
void setHandicap9(int handicap9);
void setHandicap13(int handicap13);
void setHandicap19(int handicap19);

void opponentpass();
void estimatescore();

#endif // ENGINE_H
